package xvs.ACL;

import java.io.IOException;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

import android.hardware.Camera;
import android.hardware.Camera.AutoFocusCallback;
import android.hardware.Camera.Parameters;
import android.hardware.Camera.Size;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.ViewGroup;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ActivityObject;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.Permissions;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;

/**
 *The new, improved camera library has a lot of new functions!
 *These include: flashlight, scene modes, orientations, front- and back camera, picture size, white balance and a lot more!
 *Only one process can access the camera at any time. Therefore it is highly recommended to initialize the camera object in Activity_Resume and release it in Activity_Pause.
 *A working example with explanations is available <link>here|http://www.basic4ppc.com/forum/basic4android-getting-started-tutorials/6891-take-pictures-internal-camera.html</link>.
 */
@ActivityObject
@Permissions(values={"android.permission.CAMERA","android.permission.FLASHLIGHT"})
@ShortName("AdvancedCamera")
@Version(2.0f)
@Author("XverhelstX")
@Events(values={"Ready (Success As Boolean)", "PictureTaken (Data() As Byte)"})
public class ACL {
    private static Camera c;
    private SurfaceView sv;
    private String eventName;
    private BA ba;
    private AtomicInteger readyCount = new AtomicInteger();

   
    /**
     * Initializes the camera.
     *Panel - The preview images will be displayed on the panel.
     *EventName - Events subs prefix.
     *The Ready event will be raised when the camera has finished opening.
     */
    @SuppressWarnings("static-access")
	public void Initialize(final BA ba, ViewGroup Panel, String EventName) {
        this.ba = ba;
        readyCount.set(0);
        this.eventName = EventName.toLowerCase(BA.cul);
        sv = new SurfaceView(ba.context);
        anywheresoftware.b4a.BALayout.LayoutParams lp = new anywheresoftware.b4a.BALayout.LayoutParams(0, 0,
                Panel.getLayoutParams().width, Panel.getLayoutParams().height);
        Panel.addView(sv, lp);
        if (c != null) {
            readyCount.set(1);
        }
    	
     
        sv.getHolder().setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
        sv.getHolder().setFixedSize(Panel.getLayoutParams().width, Panel.getLayoutParams().height);
        sv.getHolder().addCallback(new SurfaceHolder.Callback() {

            @Override
            public void surfaceChanged(SurfaceHolder holder, int format,
                    int width, int height) {
            	
            	
            
            }

            @Override
            public void surfaceCreated(SurfaceHolder holder) {
                if (readyCount.addAndGet(1) == 2) {
                    ba.raiseEvent(null, eventName + "_ready", true);
                }
            }

            @Override
            public void surfaceDestroyed(SurfaceHolder holder) {

            }

        });
        if (c == null) {
            ba.submitRunnable(new Runnable() {

                @Override
                public void run() {
                    try {
                        c = Camera.open();
                        if (readyCount.addAndGet(1) == 2) {
                            ba.raiseEventFromDifferentThread(null, ACL.this, -1,eventName +  "_ready", false, new Object[] {true});
                        }
                    }
                    catch (Exception e) {
                        Release();
                        ba.raiseEventFromDifferentThread(null, ACL.this, -1,eventName +  "_ready", false, new Object[] {false});
                    }
                }

            }, this, -1);
        }
    }
    
    /**
     * Starts displaying the preview images.
     */
    public void StartPreview() throws IOException {
    	
        c.setPreviewDisplay(sv.getHolder());
        c.startPreview();
    }
    /**
     * Stops displaying the preview images.
     */
    public void StopPreview() {
        if (c != null)
            c.stopPreview();
    }
    /**
     * Releases the camera object and allows other processes to access the camera.
     */
    public void Release() {
        if (sv != null) {
            ViewGroup vg = (ViewGroup) sv.getParent();
            vg.removeView(sv);
            sv = null;
        }
        if (c != null) {
            c.release();
            c = null;
        }
    }
    /**
     * Takes a picture. When the picture is ready, the PictureTaken event will be raised.
     *You should not call TakePicture while another picture is currently taken.
     *The preview images are stopped after calling this method. You can call StartPreview to restart the preview images.
     */
    public void TakePicture() {
        c.takePicture(null , null, new Camera.PictureCallback() {

            @Override
            public void onPictureTaken(byte[] data, Camera camera) {
                ba.raiseEvent(null, eventName + "_picturetaken", data);
            }
            
        });
    }
   
    /**
     * Use the back camera as primary.
     */

	public void CameraBack(){
    	Camera.Parameters params = c.getParameters();
    	params.set("camera-id", 1);
    	c.setParameters(params);
    
        
    }
    
    /**
     * Use the front camera as primary.
     */
	public void CameraFront(){
    	Camera.Parameters params = c.getParameters();
    	params.set("camera-id", 2);
    	c.setParameters(params);
        
    }
    
    /**
     * Sets the camera orientation to portrait.
     */
    public void OriPortrait() {
    	Camera.Parameters params = c.getParameters();
    	c.setDisplayOrientation(90);
    	c.setParameters(params);
    	
    }
    
    /**
     * Sets the camera orientation to landscape.
     */
    public void OriLandscape() {
    	Camera.Parameters params = c.getParameters();
    	c.setDisplayOrientation(0);
    	c.setParameters(params);
    	
    }
    /**
     * Sets the exposure compensation.
     * A technique for adjusting the exposure indicated by a photographic exposure meter.
     */
    public void setsetExposureCompensation (int value){
    	Camera.Parameters params = c.getParameters();
    	params.setExposureCompensation(value);
    	c.setParameters(params);
    	
    }
    
    /**
     * Sets the iso value.
     * Note that this MIGHT not work (properly).
     */
    public void setsetISOValue (int value){
    Camera.Parameters params = c.getParameters(); 
    params.set("iso", value);
    c.setParameters(params);
    }
    
    /**
     * Sets the size of the picture in pixels.
     */
    public void PictureSize(int width, int height) {
    	Camera.Parameters params = c.getParameters();
    	params.setPictureSize(width, height);
    	c.setParameters(params);

    }
    
    /**
     * Sets the zooming level.
     */
    public void setsetZoom(int value){
    	Camera.Parameters params = c.getParameters();
    	String zoomSupportedString = params.get("zoom-supported");
        if (zoomSupportedString != null && !Boolean.parseBoolean(zoomSupportedString)) {
        	params.setZoom(value);
        	c.setParameters(params);
        }


      }
    
    /**
     * Returns the maximum zooming level.
     * 0 is no zooming supported.
     * @return 
     */
    public int getgetMaxZoom(){
    	Camera.Parameters params = c.getParameters();
    	return params.getMaxZoom();
        }
    
    /**
     * Returns the maximum exposure compensation.
     * @return 
     */
    public int getgetMaxExposureCompensation(){
    	Camera.Parameters params = c.getParameters();
    	return params.getMaxExposureCompensation();
    }
    
    /**
     * Returns the minimum exposure compensation.
     * @return 
     */
    public int getgetMinExposureCompensation(){
    	Camera.Parameters params = c.getParameters();
    	return params.getMinExposureCompensation();
    }
    
    /**
     * Returns a list of all supported colour effects.
     * @return 
     */
    public List<String> getgetSupportedColourEffect(){
    	Camera.Parameters params = c.getParameters();
    	return params.getSupportedColorEffects();
    }
    
    /**
     * Returns the current colour effect. (String)
     * @return 
     */
    public String getgetCurrentColourEffect(){
    	Camera.Parameters params = c.getParameters();
    	return params.getColorEffect();
    }

    /**
     * Returns a list of all supported focus modes.
     * @return 
     */
    public List<String> getgetSupportedFocusMode(){
    	Camera.Parameters params = c.getParameters();
    	return params.getSupportedFocusModes();
    }
    
    /**
     * Returns the current focus mode. (String)
     * @return 
     */
    public String getgetCurrentFocusMode(){
    	Camera.Parameters params = c.getParameters();
    	return params.getFocusMode();
    }
    
    /**
     * Returns a list of all supported picture sizes.
     * @return 
     */
    public List<Size> getgetSupportedPictureSize(){
    	Camera.Parameters params = c.getParameters();
    	return params.getSupportedPictureSizes();
    }
    
    /**
     * Returns the current picture size. (Size)
     * @return 
     */
    public Size getgetCurrentPictureSize(){
    	Camera.Parameters params = c.getParameters();
    	return params.getPictureSize();
    }
    
    
    /**
     * Returns a list of all supported scene modes.
     * @return 
     */
    public List<String> getgetSupportedSceneMode(){
    	Camera.Parameters params = c.getParameters();
    	return params.getSupportedSceneModes();
    }
    
    /**
     * Returns the current scene mode. (String)
     * @return 
     */
    public String getgetCurrentSceneMode(){
    	Camera.Parameters params = c.getParameters();
    	return params.getSceneMode();
    }
    
    
    /**
     * Returns a list of all supported white balances.
     * @return 
     */
    public List<String> getgetSupportedWhiteBalance(){
    	Camera.Parameters params = c.getParameters();
    	return params.getSupportedWhiteBalance();
    }
    
    /**
     * Returns the current white balance. (String)
     * @return 
     */
    public String getgetCurrentWhiteBalance(){
    	Camera.Parameters params = c.getParameters();
    	return params.getWhiteBalance();
    }
    
    /**
     * Returns a list of all supported flash modes.
     * @return 
     */
    public List<String> getgetSupportedFlashMode(){
    	Camera.Parameters params = c.getParameters();
    	return params.getSupportedFlashModes();
    }
    
    /**
     * Returns the current flash mode. (String)
     * @return 
     */
    public String getgetCurrentFlashMode(){
    	Camera.Parameters params = c.getParameters();
    	return params.getFlashMode();
    }
    
    /**
     * Returns a list of all supported antibandings.
     * @return 
     */
    public List<String> getgetSupportedAntibanding(){
    	Camera.Parameters params = c.getParameters();
    	return params.getSupportedAntibanding();
    }
    
    /**
     * Returns the current antibanding. (String)
     * @return 
     */
    public String getgetCurrentAntibanding(){
    	Camera.Parameters params = c.getParameters();
    	return params.getAntibanding();
    }
    
    /**
     * Sets the quality of the JPEG file.
     * The range is between 1 to 100, with 100 being the best.
     */
    public void setQuality(int quality) {
    	Camera.Parameters params = c.getParameters();
    	params.setJpegQuality(quality);
    	c.setParameters(params);

    }
    
    /**
     * Returns a true or false statement when zooming is supported or not.
     * @return 
     */
    public boolean isZoomSupported(){
    	Camera.Parameters params = c.getParameters();
    	String zoomSupportedString = params.get("zoom-supported");
        if (zoomSupportedString != null && !Boolean.parseBoolean(zoomSupportedString)) {
        	return false;
        }
        else
        {
		return true;
        }
		
        }
    
    
    /**
     *Enables the flashlight to turn on automatically.
     */
    public void FlashAuto() {
    	Camera.Parameters params = c.getParameters();
        params.setFlashMode( Camera.Parameters.FLASH_MODE_AUTO );
        c.setParameters(params);    
    }
    
    /**
     *Turns the flashlight on.
     */
    public void FlashOn() {
    	Camera.Parameters params = c.getParameters();
        params.setFlashMode( Camera.Parameters.FLASH_MODE_TORCH );
        c.setParameters(params);    
    }
    
    /**
     *Turns the flashlight off.
     */
    public void FlashOff() {
    	Camera.Parameters params = c.getParameters();
        params.setFlashMode( Camera.Parameters.FLASH_MODE_OFF );
        c.setParameters(params);  
    }
        
    /**
     *Sets the focus mode.
     *MACRO: Close-up, INFINITY: Far distance.
     *FIXED: If the camera has auto-focus, this mode can fix the focus.
     *AUTO: Auto-focus mode, EDOF: Focusing is done digitally and continuously.
     *For example:
     *Camera1.setFocusMode = "MACRO"
     *
     */
    public void setsetFocusMode(String Focusmode) {
    	
    	Camera.Parameters params = c.getParameters();
    	if (Focusmode == "MACRO"){
    		c.autoFocus(mAutofocusCallback);
        	params.setFocusMode( Parameters.FOCUS_MODE_MACRO );
            c.setParameters(params);  
    	}
    	if (Focusmode == "INFINITY"){
            params.setFocusMode( Parameters.FOCUS_MODE_INFINITY );
            c.setParameters(params);   
    	}
    	if (Focusmode == "FIXED"){
            params.setFocusMode( Parameters.FOCUS_MODE_FIXED );
            c.setParameters(params);  
    	}
    	if (Focusmode == "AUTO"){
    		c.autoFocus(mAutofocusCallback);
            params.setFocusMode( Camera.Parameters.FOCUS_MODE_AUTO );
            c.setParameters(params);  
    	}
    	if (Focusmode == "EDOF"){
            params.setFocusMode( Camera.Parameters.FOCUS_MODE_EDOF );
            c.setParameters(params);  
    	}
    	
     
    }
    
    AutoFocusCallback mAutofocusCallback = new AutoFocusCallback(){

		@Override
		public void onAutoFocus(boolean arg0, Camera arg1) {
			// TODO Auto-generated method stub
			
		}};
    /**
     *Sets the Scene Mode.
     *ACTION: Take photos of fast moving objects., AUTO: Scene mode is off.
     *BEACH: Take pictures on the beach
     *CANDLELIGHT: Capture scenes lit by candles, FIREWORKS: Take pictures of fireworks.
     *LANDSCAPE: Take pictures on distant objects., NIGHT: Take photos at night.
     *PARTY: Indoor low-light shot., PORTRAIT: Take people pictures.
     *SNOW: in the snow., STEADYPHOTO: Avoid blurry pictures (handshake).
     *SUNSET: Take Sunset photos., THEATRE: Take photos in a theater.
     *For example:
     *Camera1.setScene = "BEACH"
     *
     */
    public void setsetSceneMode(String SceneMode) {
    	Camera.Parameters params = c.getParameters();
    	if (SceneMode == "ACTION"){
    		params.setSceneMode(Camera.Parameters.SCENE_MODE_ACTION);
    		c.setParameters(params);  
    	}
    	if (SceneMode == "AUTO"){
    		params.setSceneMode(Camera.Parameters.SCENE_MODE_AUTO);
    		c.setParameters(params);  
    	}
    	if (SceneMode == "BEACH"){
    		params.setSceneMode(Camera.Parameters.SCENE_MODE_BEACH);
    		c.setParameters(params);  
    	}
    	if (SceneMode == "CANDLELIGHT"){
    		params.setSceneMode(Camera.Parameters.SCENE_MODE_CANDLELIGHT);
    		c.setParameters(params);  
    	}
    	if (SceneMode == "FIREWORKS"){
    		params.setSceneMode(Camera.Parameters.SCENE_MODE_FIREWORKS);
    		c.setParameters(params);  
    	}
    	if (SceneMode == "LANDSCAPE"){
    		params.setSceneMode(Camera.Parameters.SCENE_MODE_LANDSCAPE);
    		c.setParameters(params);  
    	}
    	if (SceneMode == "NIGHT"){
    		params.setSceneMode(Camera.Parameters.SCENE_MODE_NIGHT);
    		c.setParameters(params);  
    	}
    	if (SceneMode == "PARTY"){
    		params.setSceneMode(Camera.Parameters.SCENE_MODE_PARTY);
    		c.setParameters(params);  
    	}
    	if (SceneMode == "PORTRAIT"){
    		params.setSceneMode(Camera.Parameters.SCENE_MODE_PORTRAIT);
    		c.setParameters(params);  
    	}
    	if (SceneMode == "SNOW"){
    		params.setSceneMode(Camera.Parameters.SCENE_MODE_SNOW);
    		c.setParameters(params);  
    	}
    	if (SceneMode == "STEADYPHOTO"){
    		params.setSceneMode(Camera.Parameters.SCENE_MODE_STEADYPHOTO);
    		c.setParameters(params);  
    	}
    	if (SceneMode == "SUNSET"){
    		params.setSceneMode(Camera.Parameters.SCENE_MODE_SUNSET);
    		c.setParameters(params);  
    	}
    	if (SceneMode == "THEATRE"){
    		params.setSceneMode(Camera.Parameters.SCENE_MODE_THEATRE);
    		c.setParameters(params);  
    	}
    }
    	
    	/**
         *Sets the Antibanding.
         *ANTIBANDING_50HZ: Sets the antibanding to 50Hz.
         *ANTIBANDING_60HZ: Sets the antibanding to 60Hz.
         *ANTIBANDING_AUTO: Sets the antibanding to automatic mode.
         *ANTIBANDING_OFF: Sets the antibanding off.
         *For example:
         *Camera1.setAntibanding = "ANTIBANDING_AUTO"
         */
        public void setsetAntibanding(String Antibanding) {
        	Camera.Parameters params = c.getParameters();
        	if (Antibanding == "ANTIBANDING_50HZ"){
        		params.setAntibanding(Camera.Parameters.ANTIBANDING_50HZ);
        		c.setParameters(params);  
        	}
        	if (Antibanding == "ANTIBANDING_60HZ"){
        		params.setAntibanding(Camera.Parameters.ANTIBANDING_60HZ);
        		c.setParameters(params);  
        	}
        	if (Antibanding == "ANTIBANDING_AUTO"){
        		params.setAntibanding(Camera.Parameters.ANTIBANDING_AUTO);
        		c.setParameters(params);  
        	}
        	if (Antibanding == "ANTIBANDING_OFF"){
        		params.setAntibanding(Camera.Parameters.ANTIBANDING_OFF);
        		c.setParameters(params);  
        	}
        }
        	
            /**
             *Sets the color effect of a picture.
             *The following effects are: AQUA, BLACKBOARD, MONO,
             *NEGATIVE, NONE, POSTERIZE, SEPIA, SOLARIZE, WHITEBOARD.
             *For example:
             *Camera1.setColourEffect = "SEPIA"
             */
            public void setsetColourEffect(String Effect) {
            	Camera.Parameters params = c.getParameters();
            	if (Effect == "AQUA"){
            		params.setColorEffect(Camera.Parameters.EFFECT_AQUA);
            		c.setParameters(params);  
            	}
            	if (Effect == "BLACKBOARD"){
            		params.setColorEffect(Camera.Parameters.EFFECT_BLACKBOARD);
            		c.setParameters(params);  
            	}
            	if (Effect == "MONO"){
            		params.setColorEffect(Camera.Parameters.EFFECT_MONO);
            		c.setParameters(params);  
            	}
            	if (Effect == "NEGATIVE"){
            		params.setColorEffect(Camera.Parameters.EFFECT_NEGATIVE);
            		c.setParameters(params);  
            	}
            	if (Effect == "NONE"){
            		params.setColorEffect(Camera.Parameters.EFFECT_NONE);
            		c.setParameters(params);  
            	}
            	if (Effect == "POSTERIZE"){
            		params.setColorEffect(Camera.Parameters.EFFECT_POSTERIZE);
            		c.setParameters(params);  
            	}
            	if (Effect == "SEPIA"){
            		params.setColorEffect(Camera.Parameters.EFFECT_SEPIA);
            		c.setParameters(params);  
            	}
            	if (Effect == "SOLARIZE"){
            		params.setColorEffect(Camera.Parameters.EFFECT_SOLARIZE);
            		c.setParameters(params);  
            	}
            	if (Effect == "WHITEBOARD"){
            		params.setColorEffect(Camera.Parameters.EFFECT_WHITEBOARD);
            		c.setParameters(params);  
            	}
            }
            	
                /**
                 *Sets the white balance.
                 *The following effects are: AUTO, CLOUDY_DAYLIGHT, DAYLIGHT,
                 *FLUORESCENT, INCANDESCENT, SHADE, TWILIGHT, WARM_FLUORESCENT.
                 *For example:
                 *Camera1.setEffect = "WARM_FLUORESCENT"
                 */
                public void setsetWhiteBalance(String Balance) {
                	Camera.Parameters params = c.getParameters();
                	if (Balance == "AUTO"){
                		params.setWhiteBalance(Camera.Parameters.WHITE_BALANCE_AUTO);
                		c.setParameters(params);  
                	}
                	if (Balance == "CLOUDY_DAYLIGHT"){
                		params.setWhiteBalance(Camera.Parameters.WHITE_BALANCE_CLOUDY_DAYLIGHT);
                		c.setParameters(params);  
                	}
                	if (Balance == "DAYLIGHT"){
                		params.setWhiteBalance(Camera.Parameters.WHITE_BALANCE_DAYLIGHT);
                		c.setParameters(params);  
                	}
                	if (Balance == "FLUORESCENT"){
                		params.setWhiteBalance(Camera.Parameters.WHITE_BALANCE_FLUORESCENT);
                		c.setParameters(params);  
                	}
                	if (Balance == "INCANDESCENT"){
                		params.setWhiteBalance(Camera.Parameters.WHITE_BALANCE_INCANDESCENT);
                		c.setParameters(params);  
                	}
                	if (Balance == "SHADE"){
                		params.setWhiteBalance(Camera.Parameters.WHITE_BALANCE_SHADE);
                		c.setParameters(params);  
                	}
                	if (Balance == "TWILIGHT"){
                		params.setWhiteBalance(Camera.Parameters.WHITE_BALANCE_TWILIGHT);
                		c.setParameters(params);  
                	}
                	if (Balance == "WARM_FLUORESCENT"){
                		params.setWhiteBalance(Camera.Parameters.WHITE_BALANCE_WARM_FLUORESCENT);
                		c.setParameters(params);  
                	}
        	
        	
    }
                
              

}